﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using PluginDeploymentService = HeuristicLab.PluginInfrastructure.Advanced.DeploymentService;

namespace HeuristicLab.DeploymentService.AdminClient {
  public partial class ProductEditor : HeuristicLab.MainForm.WindowsForms.View {
    private BackgroundWorker refreshProductsWorker;
    private BackgroundWorker uploadChangedProductsWorker;
    private List<PluginDeploymentService.ProductDescription> products;
    private List<PluginDeploymentService.PluginDescription> plugins;
    private HashSet<PluginDeploymentService.ProductDescription> dirtyProducts;

    public ProductEditor() {
      InitializeComponent();
      Caption = "Products";

      dirtyProducts = new HashSet<PluginDeploymentService.ProductDescription>();
      refreshProductsWorker = new BackgroundWorker();
      refreshProductsWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(refreshProductsWorker_RunWorkerCompleted);
      refreshProductsWorker.DoWork += new DoWorkEventHandler(refreshProductsWorker_DoWork);

      uploadChangedProductsWorker = new BackgroundWorker();
      uploadChangedProductsWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(uploadChangedProductsWorker_RunWorkerCompleted);
      uploadChangedProductsWorker.DoWork += new DoWorkEventHandler(uploadChangedProductsWorker_DoWork);
    }

    void uploadChangedProductsWorker_DoWork(object sender, DoWorkEventArgs e) {
      var products = (IEnumerable<PluginDeploymentService.ProductDescription>)e.Argument;
      using (var adminClient = new PluginDeploymentService.AdminClient()) {
        foreach (var product in products) {
          adminClient.DeployProduct(product);
        }
      }
      e.Cancel = false;
    }

    void uploadChangedProductsWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
      this.Enabled = true;
    }

    void refreshProductsWorker_DoWork(object sender, DoWorkEventArgs e) {
      var updateClient = new PluginDeploymentService.UpdateClient();
      e.Result = new object[] { updateClient.GetProducts(), updateClient.GetPlugins() };
    }

    void refreshProductsWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
      this.products = new List<PluginDeploymentService.ProductDescription>(
        (PluginDeploymentService.ProductDescription[])((object[])e.Result)[0]);
      this.plugins = new List<PluginDeploymentService.PluginDescription>(
        (PluginDeploymentService.PluginDescription[])((object[])e.Result)[1]);

      UpdateProductsList();
      dirtyProducts.Clear();

      Cursor = Cursors.Default;
      SetControlsEnabled(true);
    }

    private void UpdateProductsList() {
      foreach (var prodDesc in products) {
        productsListBox.Items.Add(CreateListViewItem(prodDesc));
      }
    }

    private void productsListBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (productsListBox.SelectedItems.Count == 0) return;
      PluginDeploymentService.ProductDescription activeProduct = (PluginDeploymentService.ProductDescription)((ListViewItem)productsListBox.SelectedItem).Tag;
      UpdateProductDetails(activeProduct);
    }

    private void UpdateProductDetails(PluginDeploymentService.ProductDescription activeProduct) {
      nameTextBox.Text = activeProduct.Name;
      versionTextBox.Text = activeProduct.Version.ToString();

      pluginsList.Items.Clear();
      foreach (PluginDeploymentService.PluginDescription pluginDesc in plugins) {
        var matching = from p in activeProduct.Plugins
                       where p.Name == pluginDesc.Name
                       where p.Version == pluginDesc.Version
                       select p;

        if (matching.Any()) pluginsList.Items.Add(pluginDesc, true);
        else pluginsList.Items.Add(pluginDesc, false);
      }
    }

    private void refreshButton_Click(object sender, EventArgs e) {
      SetControlsEnabled(false);
      Cursor = Cursors.AppStarting;
      refreshProductsWorker.RunWorkerAsync();
    }

    private void SetControlsEnabled(bool enabled) {
      saveButton.Enabled = enabled;
      refreshButton.Enabled = enabled;
      splitContainer.Enabled = enabled;
    }

    private void newProductButton_Click(object sender, EventArgs e) {
      var newProduct = new PluginDeploymentService.ProductDescription("New product", new Version("0.0.0.0"));
      ListViewItem item = CreateListViewItem(newProduct);
      productsListBox.Items.Add(item);
      dirtyProducts.Add(newProduct);
    }

    private ListViewItem CreateListViewItem(PluginDeploymentService.ProductDescription productDescription) {
      ListViewItem item = new ListViewItem();
      item.Text = productDescription.Name + " " + productDescription.Version;
      item.Name = productDescription.Name + " " + productDescription.Version;
      item.Tag = productDescription;
      return item;
    }

    private void saveButton_Click(object sender, EventArgs e) {
      this.Enabled = false;
      uploadChangedProductsWorker.RunWorkerAsync(dirtyProducts);
    }

    private void nameTextBox_TextChanged(object sender, EventArgs e) {
      ListViewItem activeItem = (ListViewItem)productsListBox.SelectedItem;
      PluginDeploymentService.ProductDescription activeProduct = (PluginDeploymentService.ProductDescription)activeItem.Tag;
      if (string.IsNullOrEmpty(nameTextBox.Name)) {
        errorProvider.SetError(nameTextBox, "Invalid value");
      } else {
        activeProduct.Name = nameTextBox.Text;
        activeItem.Text = activeProduct.Name + " " + activeProduct.Version;
        activeItem.Name = activeProduct.Name + " " + activeProduct.Version;
        errorProvider.SetError(nameTextBox, string.Empty);
      }
    }

    private void versionTextBox_TextChanged(object sender, EventArgs e) {
      ListViewItem activeItem = (ListViewItem)productsListBox.SelectedItem;
      PluginDeploymentService.ProductDescription activeProduct = (PluginDeploymentService.ProductDescription)activeItem.Tag;
      try {
        activeProduct.Version = new Version(versionTextBox.Text);
        activeItem.Text = activeProduct.Name + " " + activeProduct.Version;
        activeItem.Name = activeProduct.Name + " " + activeProduct.Version;
        errorProvider.SetError(versionTextBox, string.Empty);
      }
      catch (OverflowException ex) {
        errorProvider.SetError(versionTextBox, "Invalid value");
      }

      catch (ArgumentException ex) {
        errorProvider.SetError(versionTextBox, "Invalid value");
      }
      catch (FormatException ex) {
        errorProvider.SetError(versionTextBox, "Invalid value");
      }
    }
  }
}
